# (c) Copyright 2012, CodeWeavers, Inc.

import os
import sys
import traceback

import gtk
gtk.gdk.threads_init()

import gobject
import gtk.glade
import pango

import bottlequery
import bottletreeview
import cxlog
import cxmenu
import cxutils

import bottlecollection
import cxguitools


import cxprefs
import cxsetup
import installwizard
import cxregister

from cxutils import cxgettext as _

class CrossOver:
    """This is the CrossOver application"""

    prefController = None
    bottleController = None
    installWizard = None
    registrationController = None
    ccenter = None

    def __init__(self):

        # Load & show the main CrossOver window

        try:
            gladepath = os.environ["CX_GLADEPATH"]
        except KeyError:
            gladepath = os.path.join(cxutils.CX_ROOT, "lib", "python", "glade")

        self.gladefile = os.path.join(gladepath,"crossover.glade")
        
        self.bottles = bottlecollection.sharedCollection(('basic',))
        self.bottles.addChangeDelegate(self)

        if not gtk.glade.textdomain() == "crossover":
            locale_path = os.path.join(cxutils.CX_ROOT, "share", "locale")
            gtk.glade.bindtextdomain("crossover", locale_path)
            gtk.glade.textdomain("crossover")

        self.mainWindowXML = gtk.glade.XML(self.gladefile, "MainWindow")

        self.mainWindowXML.signal_autoconnect(self)

        self.mainWindow = self.mainWindowXML.get_widget("MainWindow")

        # do some fussing with the window layout before drawing.

        iconImage = self.mainWindowXML.get_widget("CrossOverIconImage")
        logoImage = self.mainWindowXML.get_widget("CrossOverLogoImage")

        iconImage.set_from_file(os.path.join(cxutils.CX_ROOT,
                                             "share", 
                                             "images",
                                             "welcomeCrossOverIcon.png"))

        logoImage.set_from_file(os.path.join(cxutils.CX_ROOT,
                                             "share", 
                                             "images",
                                             "welcomeCrossOverLogo.png"))

        windowsSoftwareScrolledWindow = self.mainWindowXML.get_widget("WindowsSoftwareScrolledWindow")
        softwareListColumnVBox = self.mainWindowXML.get_widget("SoftwareListColumnVBox")
        
        windowsSoftwareList = self._getInstalledWindowsSoftware()

        if windowsSoftwareList:
            self._populateWindowsSoftwareWindow(windowsSoftwareScrolledWindow,
                                                windowsSoftwareList,
                                                softwareListColumnVBox)
        else:
            self._createEmptyWindowsSoftwareWindow(softwareListColumnVBox)
            windowsSoftwareScrolledWindow.set_shadow_type(gtk.SHADOW_NONE)

        self.updateAppList()
        self.mainWindow.show_all()

    def updateAppList(self):
        windowsSoftwareScrolledWindow = self.mainWindowXML.get_widget("WindowsSoftwareScrolledWindow")
        softwareListColumnVBox = self.mainWindowXML.get_widget("SoftwareListColumnVBox")
        
        windowsSoftwareList = self._getInstalledWindowsSoftware()


        # clear things out before we refresh
        for child in windowsSoftwareScrolledWindow.get_children():
            windowsSoftwareScrolledWindow.remove(child)

        for child in softwareListColumnVBox.get_children():
            if windowsSoftwareList:
                if child is not windowsSoftwareScrolledWindow:
                    softwareListColumnVBox.remove(child)
            else:
                softwareListColumnVBox.remove(child)                


        if windowsSoftwareList:
            self._populateWindowsSoftwareWindow(windowsSoftwareScrolledWindow,
                                                windowsSoftwareList,
                                                softwareListColumnVBox)
        else:
            self._createEmptyWindowsSoftwareWindow(softwareListColumnVBox)
            windowsSoftwareScrolledWindow.set_shadow_type(gtk.SHADOW_NONE)

        self.mainWindow.show_all()

    def bottleCollectionChanged(self):
        self.updateAppList()

    def on_ShowHelp_activate(self, _unused):
        cxutils.launch_url(os.path.join(cxutils.CX_ROOT, "doc", "index.html"))

    def on_DoRegistration_activate(self, _unused):
        if self.registrationController is None:
            self.registrationController = cxregister.RegisterDialogController(self)
        else:
            self.registrationController.present()

    def on_MainWindow_delete_event(self, *_args):
        self.mainWindow.destroy()
        cxguitools.toplevel_quit()

    def on_ManageBottlesMenuItem_activate(self, _unused):
        if self.bottleController is None:
            self.bottleController = cxsetup.BottleManagerController()
            self.bottleController.add_observer(cxsetup.BOTTLE_MANAGER_CLOSED, self.bottleManagerDidClose)
        else:
            self.bottleController.present()

    def on_PreferencesMenuItem_activate(self, _unused):
        if self.prefController is None:
            self.prefController = cxprefs.PreferenceDialogController(self)
        else:
            self.prefController.present()

    def registerDialogDidClose(self):
        self.registrationController = None

    def on_InstallWindowsSoftware_clicked(self, _unused):
        if self.installWizard is None:
            self.installWizard = installwizard.InstallerAssistantController()
            self.installWizard.add_observer(installwizard.INSTALL_FINISHED, self.installWizardDidFinishInstalling)
            self.installWizard.add_observer(installwizard.INSTALL_WIZARD_CLOSED, self.installWizardDidClose)
        else:
            self.installWizard.present()

    def installWizardDidFinishInstalling(self, _event=None, _data=None, *_args):
        self.updateAppList()

    def installWizardDidClose(self, _event=None, _data=None, *_args):
        self.installWizard = None

    def on_QuitMenuItem_activate(self, _unused):
        self.mainWindow.destroy()
        cxguitools.toplevel_quit()

    def ccenterClosed(self):
        self.ccenter = None

    def bottleManagerDidClose(self, _event=None, _data=None, *_args):
        self.bottleController = None

    def prefsDialogDidClose(self):
        self.prefController = None

    def on_seeAllWindowsApps_clicked(self, _unused):
        if not self.ccenter:
            self.ccenter = ControlCenter(self)

    def _getInstalledWindowsSoftware(self):
        # Returns a list of windows applications which are installed,
        # or an empty list if none are found.
        #
        # The list we return is flattened across all bottles.
        # (this might not be appropriate for all purposes).

        installedApps = {}
        
        bottleList = bottlequery.get_bottle_list()

        for bottle in bottleList:
            prefs = cxmenu.MenuPrefs(bottle, True)
            prefs.read_config()
            
            for app in prefs.data:
                if(prefs.data[app].include_in_mainmenu):
                    if(prefs.data[app].include_in_mainmenu == 'alwaysinclude'):
                        installedApps[os.path.basename(app)] = prefs.data[app]
                    elif(prefs.data[app].include_in_mainmenu ==
                         'neverinclude'):
                        continue
                else:
                    if app not in installedApps:
                        installedApps[os.path.basename(app)] = prefs.data[app]

        return installedApps


    def _createEmptyWindowsSoftwareWindow(self, blankContainer):
        # Populates the windows software scrolled window with a simple 
        # bit of text directing the user to install some windows apps
        
        aVBox = gtk.VBox(False, 1)
        
        anHRule = gtk.HSeparator()
        aVBox.pack_start(anHRule)

        pleaseInstallSoftwareLabel = gtk.Label(_("\n\n\n\nYou have not installed any\nWindows software yet.\n\nPlease click the button labeled\n'Install Windows Software'\nto get started."))
        
        aVBox.pack_start(pleaseInstallSoftwareLabel)
        
        aVBox.set_child_packing(anHRule, False, True, 0, gtk.PACK_START)
        aVBox.set_child_packing(pleaseInstallSoftwareLabel, True, True, 0, gtk.PACK_START)

        blankContainer.pack_end(aVBox)


    def _populateWindowsSoftwareWindow(self, 
                                       scrolledWindow,
                                       installedApps,
                                       softwareListColumn):
        # Add programs to the programs list

        # Set up the scroll window so we can add items
        
        windowsSoftwareTable = gtk.Table(4, 1, True)
        windowsSoftwareTable.set_homogeneous(True)
        windowsSoftwareTable.set_row_spacings(5)
        windowsSoftwareTable.set_col_spacings(5)

        scrolledWindow.add_with_viewport(windowsSoftwareTable)

        self.buttonDict = {}

        vpos = 0
        for app in installedApps.keys():
            aButton = self.createButton("\n%s\n" %
                                        app.replace(".lnk", ""),
                                        installedApps[app].iconfile)
            aButton.connect("clicked", self.on_launchWindowsApp_clicked)
            self.buttonDict[aButton] = installedApps[app]
            windowsSoftwareTable.attach(aButton, 0, 1, vpos, vpos + 1,
                                        gtk.FILL, 0, 10)
            vpos += 1
            
        softwareListColumn.pack_start(scrolledWindow)

        aButton = gtk.Button()
        aButton.connect("clicked", self.on_seeAllWindowsApps_clicked)
        tmpLabel = gtk.Label(_("\nSee All Windows Applications\n"))
        attrs = pango.AttrList()
        attrs.change(pango.AttrWeight(pango.WEIGHT_ULTRABOLD))
        tmpLabel.set_attributes(attrs)
        myHBox = gtk.HBox()
        myHBox.pack_start(tmpLabel, True, True, 1)
        aButton.add(myHBox)
        softwareListColumn.pack_start(aButton, False, True, 10)


    def get_icon(self, icon_filename, height, width):
        icon = None
        if icon_filename:
            try:
                icon = gtk.gdk.pixbuf_new_from_file(icon_filename)
            except gobject.GError:
                cxlog.warn("couldn't load icon file %s:\n%s" % (cxlog.debug_str(icon_filename), traceback.format_exc()))
        if icon is None:
            icon = cxguitools.get_std_icon('crossover')
        if icon is not None:
            icon = icon.scale_simple(height, width, gtk.gdk.INTERP_BILINEAR)
        return icon


    def createButton(self, labelText, imagePath):
        
        myButton = gtk.Button()
        myButtonBox = gtk.HBox()
        
        myLabel = gtk.Label(labelText)
        myLabel.set_alignment(0.0, 0.5)

        myImage = gtk.Image()
        myImage.set_from_pixbuf(self.get_icon(imagePath, 48, 48))

        myButtonBox.set_spacing(8)
        myButtonBox.pack_start(myImage, False, False, 1)
        myButtonBox.pack_start(myLabel, True, True, 1)

        myButton.add(myButtonBox)

        return myButton

    def on_launchWindowsApp_clicked(self, buttonPressed):
        self.buttonDict[buttonPressed].start()


class ControlCenter:

    def __init__(self, creator):

        try:
            gladepath = os.environ["CX_GLADEPATH"]
        except KeyError:
            gladepath = os.path.join(cxutils.CX_ROOT, "lib", "python", "glade")

        self.gladefile = os.path.join(gladepath,"crossover.glade")

        self.allAppsWindowXML = gtk.glade.XML(self.gladefile, "AllAppsWindow")
        self.allAppsWindowXML.signal_autoconnect(self)
        
        self.allAppsWindow = self.allAppsWindowXML.get_widget("AllAppsWindow")
        self.allAppsWindow.show()

        self.application = creator

        # markup, sort, launcher
        self.launcherTreeStore = gtk.TreeStore(str, str, object)
        self.launcherTreeStore.set_sort_column_id(1, gtk.SORT_ASCENDING)
        self.launcherTree = self.allAppsWindowXML.get_widget('ProgramsTreeView')

        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("Launcher"), renderer, markup = 0)
        self.launcherTree.append_column(column)

        self.launcherTree.set_model(self.launcherTreeStore)

        # markup, sort, bottlename
        self.bottleListStore = gtk.ListStore(str, str, object)
        self.bottleListStore.set_sort_column_id(1, gtk.SORT_ASCENDING)
        self.bottleList = self.allAppsWindowXML.get_widget('BottlesList')

        renderer = gtk.CellRendererText()
        renderer.set_property('editable', True)
        column = gtk.TreeViewColumn(_("Bottle"), renderer, markup = 0)
        self.bottleList.append_column(column)

        self.bottleList.set_model(self.bottleListStore)

        self.bottles = bottlecollection.sharedCollection(('basic',))
        self.bottles.addChangeDelegate(self)

        self.updateBottleList()

        self.allAppsWindow.show()

    def closeControlCenter(self, _unused):
        self.allAppsWindow.destroy()
        self.bottles.removeChangeDelegate(self)
        self.application.ccenterClosed()

    def bottleCollectionChanged(self):
        self.updateBottleList()

    def updateBottleList(self):
        bottlesToAdd = set(self.bottles.bottleList())
        if len(bottlesToAdd) > 1:
            bottlesToAdd.add(None) # All Bottles

        currentBottles = set()
        i = self.bottleListStore.get_iter_first()
        while i is not None:
            currentBottles.add(self.bottleListStore.get_value(i, 2))
            i = self.bottleListStore.iter_next(i)

        for bottle in bottlesToAdd - currentBottles:
            if bottle is None:
                self.bottleListStore.append((_('<b>All Bottles</b>'),
                                             '',
                                             None))
            else:
                self.bottleListStore.append((cxutils.html_escape(bottle),
                                             bottle, bottle))

        for bottle in currentBottles - bottlesToAdd:
            i = self.bottleListStore.get_iter_first()
            while i is not None:
                if self.bottleListStore.get_value(i, 2) == bottle:
                    self.bottleListStore.remove(i)
                    break
                i = self.bottleListStore.iter_next(i)

        if bottlesToAdd:
            self.allAppsWindowXML.get_widget('AllProgramsWindow').show()

    def get_selected_bottlename(self):
        cursor = self.bottleList.get_cursor()
        if cursor is None:
            return None

        path, _column = cursor

        return self.bottleListStore.get_value(self.bottleListStore.get_iter(path), 2)

    def add_launchers(self, items, parent):

        for launcher in items:
            desc = cxutils.html_escape(launcher.description)
            desc = desc.replace(".lnk", "")
            row = self.launcherTreeStore.append(parent, (desc, 'x', launcher))
            if launcher.children:
                self.add_launchers(launcher.children, row)
                if launcher.expanded:
                    self.launcherTree.expand_to_path(self.launcherTreeStore.get_path(row))

    def on_ProgramsTreeView_row_activated(self, _aTreeView, aPath, _aColumn):
        myIter = self.launcherTreeStore.get_iter(aPath)
        aMenu = self.launcherTreeStore.get_value(myIter, 2)
        aMenu.start()

    def on_bottle_changed(self, _treeview):
        bottle = self.get_selected_bottlename()

        if bottle is None:
            return

        self.launcherTreeStore.clear()

        self.add_launchers(bottletreeview.BottleTreeView().get_items(bottle),
                           None)



def main():
    global app
    cxguitools.set_default_icon()
    # give the controller a name so it can be accessed from the console as __main__.app
    app = CrossOver()
    cxguitools.warn_if_root()
    gtk.main()
    sys.exit()


