# (c) Copyright 2009-2012. CodeWeavers, Inc.

import os

import gtk
import gtk.glade

import cxproduct
import cxutils

import cxguitools

class PreferenceDialogController(object):

    xml = None

    def __init__(self, creator = None):
        #-----------------------------------------------------------------------
        #  Setup the GUI
        #-----------------------------------------------------------------------
        cxguitools.set_default_icon()
        try:
            gladepath = os.environ["CX_GLADEPATH"]
        except KeyError:
            gladepath = os.path.join(cxutils.CX_ROOT, "lib", "python", "glade")

        self.gladefile = os.path.join(gladepath, "preferences.glade")

        if not gtk.glade.textdomain() == "crossover":
            locale_path = os.path.join(cxutils.CX_ROOT, "share", "locale")
            gtk.glade.bindtextdomain("crossover", locale_path)
            gtk.glade.textdomain("crossover")
        self.xml = gtk.glade.XML(self.gladefile)
        self.xml.signal_autoconnect(self)

        self.application = creator

        # Set initial values
        logUsage = False
        autoUpdate = False
        showUntested = False

        global_config = cxproduct.get_config()
        if global_config['CrossOver'].get('ReportWineUsage', '0') == '1':
            logUsage = True

        if global_config['OfficeSetup'].get('AutoUpdate', '0') == '1':
            autoUpdate = True

        if global_config['OfficeSetup'].get('ShowUntestedApps', '0') == '1':
            showUntested = True

        self.xml.get_widget('usageCheckbox').set_active(logUsage)
        self.xml.get_widget('downloadCheckbox').set_active(autoUpdate)
        self.xml.get_widget('showUntestedCheckbox').set_active(showUntested)

    def downloadCheckbox_toggled_cb(self, caller):
        if caller.get_active():
            cxproduct.save_setting('OfficeSetup', 'AutoUpdate', '1')
        else:
            cxproduct.save_setting('OfficeSetup', 'AutoUpdate', '0')

    def usageCheckbox_toggled_cb(self, caller):
        if caller.get_active():
            cxproduct.save_setting('CrossOver', 'ReportWineUsage', '1')
        else:
            cxproduct.save_setting('CrossOver', 'ReportWineUsage', '0')

    def untestedCheckbox_toggled_cb(self, caller):
        if caller.get_active():
            cxproduct.save_setting('OfficeSetup', 'ShowUntestedApps', '1')
        else:
            cxproduct.save_setting('OfficeSetup', 'ShowUntestedApps', '0')

    def quit_requested(self, *_args):
        self.xml.get_widget("PrefDlg").destroy()
        if self.application is not None:
            self.application.prefsDialogDidClose()
        cxguitools.toplevel_quit()

    def present(self):
        self.xml.get_widget("PrefDlg").present()
