# (c) Copyright 2009-2012. CodeWeavers, Inc.

import shutil
import stat

import os
import sys
import traceback

import gobject
import gtk
import gtk.glade
gtk.gdk.threads_init()

import cxlog
import cxutils
import cxopt
import distversion

import cxguitools
import pyop

import demoutils

# for localization
from cxutils import cxgettext as _

class RegisterDialogController(object):

    mainWindow = None
    xml = None
    statusDialog = None

    modeldialogxml = None
    animateFunctions = []

    def __init__(self, creator = None):

        try:
            gladepath = os.environ["CX_GLADEPATH"]
        except KeyError:
            gladepath = os.path.join(cxutils.CX_ROOT, "lib", "python", "glade")

        self.gladefile = os.path.join(gladepath, "registerdialog.glade")
        gtk.glade.set_custom_handler(cxguitools.CustomWidgets.custom_widget_handler)

        if not gtk.glade.textdomain() == "crossover":
            locale_path = os.path.join(cxutils.CX_ROOT, "share", "locale")
            gtk.glade.bindtextdomain("crossover", locale_path)
            gtk.glade.textdomain("crossover")
        self.xml = gtk.glade.XML(self.gladefile)
        self.xml.signal_autoconnect(self)
        self.application = creator

        try:
            icon = cxguitools.get_std_icon('cxregister')
            self.xml.get_widget("RegisterDialog").set_icon(icon)
        except gobject.GError:
            cxlog.warn("Couldn't load icon:\n%s" % traceback.format_exc())

        explanationText = _("""<body>
                             <p>To buy a full version of %(productname)s, visit:</p>
                             <p><a href="http://store.codeweavers.com">http://store.codeweavers.com</a></p>
                             <p>During your purchase you will be asked to create an account using your email
                             address and a password. Enter your email address and password below, and click
                             'Register' to register and unlock your install.</p>
                          </body>""") % {'productname': distversion.PRODUCT_NAME}
        
        self.xml.get_widget("ExplanationTextView").display_html(explanationText)

        registerButton = self.xml.get_widget("RegisterButton")
        registerButton.set_flags(gtk.CAN_DEFAULT)

        registerprogbar = self.xml.get_widget("RegistrationProgbar")
        registerprogbar.set_pulse_step(0.05)
        registerprogbar.hide()

        self.mainWindow = self.xml.get_widget('RegisterDialog')

        self.update_sensitivities(None)

        self.xml.get_widget("EmailEntry").grab_focus()


    def update_sensitivities(self, _caller):

        emailEntry = self.xml.get_widget("EmailEntry")
        passwordEntry = self.xml.get_widget("PasswordEntry")
        registerButton = self.xml.get_widget("RegisterButton")

        if len(passwordEntry.get_text()) and len(emailEntry.get_text()):
            registerButton.set_sensitive(True)
        else:
            registerButton.set_sensitive(False)
            registerButton.grab_default()

    def sensitivities_at_start_registration(self):
        emailEntry = self.xml.get_widget("EmailEntry")
        passwordEntry = self.xml.get_widget("PasswordEntry")
        registerButton = self.xml.get_widget("RegisterButton")
        cancelButton = self.xml.get_widget("CancelButton")
        registerprogbar = self.xml.get_widget("RegistrationProgbar")

        registerButton.set_sensitive(False)
        cancelButton.set_sensitive(False)
        passwordEntry.set_sensitive(False)
        emailEntry.set_sensitive(False)

        self.animateEvent = gobject.timeout_add(100, self.progbar_pulse)

        registerprogbar.show()

    def sensitivities_after_registration(self):
        emailEntry = self.xml.get_widget("EmailEntry")
        passwordEntry = self.xml.get_widget("PasswordEntry")
        registerButton = self.xml.get_widget("RegisterButton")
        cancelButton = self.xml.get_widget("CancelButton")
        registerprogbar = self.xml.get_widget("RegistrationProgbar")

        registerButton.set_sensitive(True)
        cancelButton.set_sensitive(True)
        passwordEntry.set_sensitive(True)
        emailEntry.set_sensitive(True)

        gobject.source_remove(self.animateEvent)

        registerprogbar.hide()
        self.update_sensitivities(None)

    def progbar_pulse(self):
        registerprogbar = self.xml.get_widget("RegistrationProgbar")
        registerprogbar.pulse()
        return True

    def register_clicked(self, _caller):
        emailEntry = self.xml.get_widget("EmailEntry")
        passwordEntry = self.xml.get_widget("PasswordEntry")

        self.sensitivities_at_start_registration()
        registerOp = RegisterOperation(self, emailEntry.get_text(), passwordEntry.get_text())
        pyop.sharedOperationQueue.enqueue(registerOp)

    def register_finished(self, success, errorString):
        self.sensitivities_after_registration()

        self.mainWindow = self.xml.get_widget('RegisterDialog')

        if success:
            cxguitools.CXMessageDlg(_("Registration successful."), buttons=gtk.BUTTONS_OK, response_function=self.quit_requested, parent=self.mainWindow, message_type=gtk.MESSAGE_INFO)
        else:
            cxguitools.CXMessageDlg(primary=_("Registration failed"), secondary=errorString, buttons=gtk.BUTTONS_OK, parent=self.mainWindow, message_type=gtk.MESSAGE_ERROR)

    def present(self):
        self.mainWindow.present()

    def quit_requested(self, _caller):
        self.mainWindow.destroy()
        if self.application is not None:
            self.application.registerDialogDidClose()
        cxguitools.toplevel_quit()

class RegisterOperation(pyop.PythonOperation):

    def __init__(self, inRegisterController, inLogin, inPassword):
        pyop.PythonOperation.__init__(self)
        self.registerController = inRegisterController
        self.login = inLogin
        self.password = inPassword
        self.success = False
        self.errorString = None

    def __unicode__(self):
        return "RegisterOperation with username " + self.login

    def main(self):
        self.success = False

        tempLicenseFile, tempSigFile, self.errorString = demoutils.download_and_split_license_file(self.login, self.password)
        if not tempSigFile:
            return

        (isDemo, username, _date, _licenseid, _revoked) = demoutils.demo_status(tempLicenseFile, tempSigFile)
        if isDemo:
            if not username:
                self.errorString = _("Unable to download a valid license file.")
            else:
                self.errorString = _("Your support contract is not valid for this product. You may need to purchase a support extension or an additional license.")
            os.remove(tempLicenseFile)
            os.remove(tempSigFile)
            return

        cmd = []
        cert_dir = os.path.join(cxutils.CX_ROOT, "etc")
        if not demoutils.can_register_as_current_user(cert_dir):
            cmd.extend((os.path.join(cxutils.CX_ROOT, "bin", "cxsu"), "--ignore-home"))
        cmd.extend((os.path.join(cxutils.CX_ROOT, "bin", "cxregister"),
                    "--install", tempLicenseFile, tempSigFile))
            
        retcode, _out, _err = cxutils.run(cmd, stderr=cxutils.GRAB)
        os.remove(tempLicenseFile)
        os.remove(tempSigFile)
        if retcode != 0:
            self.errorString = _("Registration failed")
        else:
            self.success = True
            self.errorString = None


    def finish(self):
        self.registerController.register_finished(self.success, self.errorString)
        pyop.PythonOperation.finish(self)


def install(src_license, src_sig):
    etc_dir = os.path.join(cxutils.CX_ROOT, "etc")
    dst_license = os.path.join(etc_dir, "license.txt")
    dst_sig = os.path.join(etc_dir, "license.sig")
    try:
        shutil.copyfile(src_license, dst_license)
        shutil.copyfile(src_sig, dst_sig)
        os.chmod(dst_license, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
        os.chmod(dst_sig, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
        return 0
    except Exception:
        e = (sys.exc_info())[1]
        cxlog.err(str(e))
        return 1

def main():
    # Parse the command line options
    opt_parser = cxopt.Parser(usage="%prog [--help]",
                              description="Registers and unlocks a demo version of CrossOver.")
    opt_parser.add_option('--install', action="store_true", dest='install', help="Installs the specified license files")
    (options, args) = opt_parser.parse_args()
    if options.install:
        if len(args) < 2:
            opt_parser.error("not enough arguments for --install")
        elif len(args) > 2:
            opt_parser.error("unexpected argument '%s'" % args[2])
        else:
            return install(args[0], args[1])
    elif args:
        opt_parser.error("unexpected argument '%s'" % args[0])

    cxguitools.set_default_icon('cxregister')

    cert_dir = os.path.join(cxutils.CX_ROOT, "etc")
    license_file = os.path.join(cert_dir, "license.txt")
    sig_file = os.path.join(cert_dir, "license.sig")

    demoStatus =  demoutils.demo_status(license_file, sig_file)

    if not demoStatus[0]:
        if demoStatus[1]:
            msg = _("This install of %(product)s has already been registered by:\n\n%(name)s\n\nIt is fully enabled. Thank you for your support!") % {'product': distversion.PRODUCT_NAME, 'name': demoStatus[1]}
        else:
            msg = _("This install of %s is already fully enabled.\n\nThank you for your support!") % distversion.PRODUCT_NAME
        cxguitools.CXMessageDlg(msg , buttons=gtk.BUTTONS_OK, response_function=sys.exit, message_type=gtk.MESSAGE_INFO)
        gtk.main()
    else:
        RegisterDialogController()
        gtk.main()
