# (c) Copyright 2009-2010. CodeWeavers, Inc.

import os
import traceback

import gobject
import gtk
import gtk.glade
gtk.gdk.threads_init()

import cxlog
import cxutils
import bottlequery

import bottlecollection
import bottlemanagement
import cxguitools
import pyop
import cxsetup

# for localization
from cxutils import cxgettext as _

class NoBottlesController(object):

    mainWindow = None
    xml = None

    modeldialogxml = None


    def __init__(self, inCommand):

        #----------------------------------------------------------------------
        #  Setup the GUI
        #----------------------------------------------------------------------

        try:
            gladepath = os.environ["CX_GLADEPATH"]
        except KeyError:
            gladepath = os.path.join(cxutils.CX_ROOT, "lib", "python", "glade")

        self.gladefile = os.path.join(gladepath,"nobottlesdlg.glade")

        if not gtk.glade.textdomain() == "crossover":
            locale_path = os.path.join(cxutils.CX_ROOT, "share", "locale")
            gtk.glade.bindtextdomain("crossover", locale_path)
            gtk.glade.textdomain("crossover")
        self.xml = gtk.glade.XML(self.gladefile)
        self.xml.signal_autoconnect(self)

        dlgWidget = self.xml.get_widget("NoBottlesDialog")

        try:
            icon = cxguitools.get_std_icon('cxrun')
            dlgWidget.set_icon(icon)
        except gobject.GError:
            cxlog.warn("Couldn't load icon:\n%s" % traceback.format_exc())


        #----------------------------------------------------------------------------
        #  Set up progress bar
        #----------------------------------------------------------------------------
        progbar = self.xml.get_widget("ProgBar")
        progbar.set_pulse_step(.05)
        progbar.hide()

        #----------------------------------------------------------------------------
        #  Set up bottle type combo
        #----------------------------------------------------------------------------
        templatewidget = self.xml.get_widget("bottletype")

        liststore = gtk.ListStore(gobject.TYPE_STRING)
        templatewidget.set_model(liststore)
        index = 0
        for template in sorted(bottlemanagement.template_list()):
            liststore.set_value(liststore.append(), 0, template)
            if template == "winxp":
                templatewidget.set_active(index)
            index += 1

        #----------------------------------------------------------------------------
        #  ...and bottle name. This part should be pretty safe since we know
        #   that there are no existing bottles.
        #----------------------------------------------------------------------------
        bottleNameWidget = self.xml.get_widget("newbottlename")
        bottleNameWidget.set_text(bottlequery.unique_bottle_name("New Bottle"))

        self.command =  inCommand
        commandWidget = self.xml.get_widget("commandlabel")
        commandWidget.set_text(self.command)

        self.commandOp = None
        self.set_widget_sensitivities(self)
        dlgWidget.show()

    def insert_text(self, caller, new_text, position, user_data):
        filtered = cxutils.bottlename_sanitize(new_text)
        if filtered != new_text:
            caller.emit_stop_by_name("insert-text")
            caller.emit("insert-text", filtered, position, user_data)

    def set_command(self, command):
        commandEntry = self.xml.get_widget("command")
        commandEntry.set_text(command)

    def cancel_clicked(self, caller):
        self.xml.get_widget("NoBottlesDialog").destroy()
        self.quit_requested(caller)

    def quit_requested(self, _caller):
        cxguitools.toplevel_quit()

    def progbar_pulse(self):
        progBar = self.xml.get_widget("ProgBar")
        progBar.pulse()
        return True

    def set_widget_sensitivities(self, _caller):
        if self.xml.get_widget("newbottlename").get_text():
            self.xml.get_widget("createandrun").set_sensitive(True)
        else:
            self.xml.get_widget("createandrun").set_sensitive(False)

    def create_bottle_and_run(self, _caller):
        dlgWidget = self.xml.get_widget("NoBottlesDialog")

        bottleNameWidget = self.xml.get_widget("newbottlename")
        self.bottlename = cxutils.string_to_unicode(bottleNameWidget.get_text())

        templatewidget = self.xml.get_widget("bottletype")
        template = templatewidget.get_active_text()

        progbar = self.xml.get_widget("ProgBar")

        newBottleOp = cxsetup.NewBottleOperation(self.bottlename, template, self, None)
        progbar.show()
        dlgWidget.set_sensitive(False)

        self.animateEvent = gobject.timeout_add(100, self.progbar_pulse)

        pyop.sharedOperationQueue.enqueue(newBottleOp)


    def AddBottleFinished(self, op):
        # This is a delegate method for the NewBottleOperation.
        if not op.exitStatus[0]:
            cxguitools.CXMessageDlg(primary=_("Could not create bottle"), secondary=op.exitStatus[1], message_type=gtk.MESSAGE_ERROR)

            gobject.source_remove(self.animateEvent)
            self.xml.get_widget("ProgBar").hide()
            dlgWidget = self.xml.get_widget("NoBottlesDialog")
            dlgWidget.set_sensitive(True)

        else:
            bottlecollection.sharedCollection().refresh()
            setDefaultBottleOp = cxsetup.SetDefaultBottleOperation(self.bottlename, self, True)
            pyop.sharedOperationQueue.enqueue(setDefaultBottleOp)

    def MakeDefaultFinished(self, _runCommandOp):
        # This is a delegate method for the SetDefaultBottleOperation. Once the bottle
        #  is created, we want to launch the requested command and exit.
        gobject.source_remove(self.animateEvent)
        args = [os.path.join(cxutils.CX_ROOT, "bin", "cxstart"),
                "--bottle", self.bottlename, "--"]
        args.extend(cxutils.cmdlinetoargv(self.command))
        os.execv(args[0], args)
        self.quit_requested(self)

