# (c) Copyright 2009. CodeWeavers, Inc.

import re

import globtree
import cxlog
import cxobjc

import c4profilesmanager


class CDDetector(cxobjc.Proxy):
    pass

@cxobjc.method(CDDetector, 'findMatchingProfilesForVolume_inProfiles_showUntested_')
def get_cd_profiles(root_path, profiles, show_untested_apps):
    product_profiles = profiles.copy()
    for profile in profiles:
        if not profiles[profile].is_for_current_product \
           or (not profiles[profile].is_ranked and not show_untested_apps):
            product_profiles.pop(profile)


    return get_cd_profiles_for_all_products(root_path, product_profiles)

@cxobjc.method(CDDetector, 'findMatchingProfilesForAllProductsForVolume_inProfiles_')
def get_cd_profiles_for_all_products(root_path, profiles):
    # Cache the FileGlobTree object
    if 'CDFileGlobTree' not in profiles.userdata:
        cd_glob = globtree.FileGlobTree()
        for profile in profiles.values():
            if profile.cd_profile:
                for glob in profile.cd_profile.globs:
                    if glob.patterns and glob.patterns != [""]:
                        patterns = []
                        for regex in glob.patterns:
                            try:
                                patterns.append(re.compile(regex))
                            except re.error:
                                cxlog.warn("cd glob pattern %s in profile %s is an invalid regular expression" % (cxlog.debug_str(regex), cxlog.debug_str(profile.appid)))
                    else:
                        patterns = ()
                    cd_glob.add_glob(glob.glob, (profile, patterns))
        profiles.userdata.setdefault('CDFileGlobTree', cd_glob)
    cd_glob = profiles.userdata['CDFileGlobTree']

    # globtree requires that the root path ends in / or is ''
    if not root_path.endswith('/'):
        root_path = '%s/' % root_path

    # Search the CD
    result = {}
    file_contents = last_path = None
    for path, (profile, patterns) in cd_glob.matches(root_path):
        if profile.appid in result:
            continue

        if patterns:
            # Read the file
            if path != last_path:
                # We take advantage of the fact that globtree will always
                # return matches for the same file all at once and avoid
                # reading e.g. autorun.inf more than once.
                last_path = path
                try:
                    # The re module doesn't seem to like CRLF so we open the
                    # file in U mode, which converts CRLF to LF.
                    infile = open(path, 'U')
                    try:
                        file_contents = infile.read()
                    finally:
                        infile.close()
                except IOError:
                    file_contents = None
                    continue
            elif file_contents is None:
                # There was an error the last time we tried to access this
                # file.
                continue

            # Check the regular expressions.
            for pattern in patterns:
                if not pattern.search(file_contents):
                    search_failed = True
                    break
            else:
                search_failed = False
            if search_failed:
                continue

        result[profile.appid] = profile

    return result


def find_profile(profile, paths):
    """Looks for the specified profile in the given list of directories."""
    profiles = c4profilesmanager.C4ProfilesSet()
    profiles[profile.appid] = profile
    matches = []
    for path in paths:
        if get_cd_profiles_for_all_products(path, profiles):
            matches.append(path)
    return matches
